var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { html } from "lit";
import { eventOptions, property, query, queryAssignedNodes, state } from "lit/decorators.js";
import { Debouncer, FtLitElement } from "@fluid-topics/ft-wc-utils";
import { FtTypography } from "@fluid-topics/ft-typography";
import { styles } from "./ft-tooltip.styles";
import { computeOffsetPosition } from "@fluid-topics/ft-wc-utils/build/floating";
class FtTooltip extends FtLitElement {
    constructor() {
        super(...arguments);
        this.text = "";
        this.manual = false;
        this.inline = false;
        this.delay = 500;
        this.position = "bottom";
        this.visible = false;
        this.validPositions = new Set(["top", "bottom", "left", "right"]);
        this.hideDebounce = new Debouncer();
        this.revealDebouncer = new Debouncer();
    }
    get validPosition() {
        return this.validPositions.has(this.position) ? this.position : "bottom";
    }
    render() {
        return html `
            <div part="container"
                 class="ft-tooltip--container ${this.inline ? "ft-tooltip--inline" : ""}"
                 @mouseenter=${this.onHover}
                 @mouseleave=${this.onOut}
                 @focusin=${this.onHover}
                 @focusout=${this.onOut}
                 @touchstart=${this.onTouch}>
                <slot part="slot" @click=${this.onClick}></slot>
                <div part="tooltip" role="tooltip" inert
                     class="ft-tooltip ft-tooltip--${this.validPosition}"
                     ?hidden=${!this.visible}>
                    <div part="tooltip-content" class="ft-tooltip--content">
                        <slot name="text"></slot>
                        <ft-typography variant="caption" ?hidden=${!this.text}>${this.text}</ft-typography>
                    </div>
                </div>
            </div>
        `;
    }
    updated(props) {
        if (props.has("visible") && this.visible) {
            this.resetTooltipContent();
        }
        super.updated(props);
    }
    contentAvailableCallback(props) {
        super.contentAvailableCallback(props);
        if (["visible", "text"].some(p => props.has(p)) && this.visible) {
            this.positionTooltip();
        }
    }
    async show(duration) {
        this.visible = true;
        if (duration != null) {
            await this.hideDebounce.run(() => {
                this.hide();
            }, duration);
        }
    }
    hide() {
        this.visible = false;
    }
    toggle() {
        this.visible = !this.visible;
    }
    get slottedElement() {
        var _a;
        return ((_a = this.slotNodes) !== null && _a !== void 0 ? _a : []).filter(node => node.nodeType == Node.ELEMENT_NODE)[0];
    }
    resetTooltipContent() {
        if (this.tooltip && this.tooltipContent) {
            const style = this.tooltipContent.style;
            style.transition = "none";
            switch (this.validPosition) {
                case "top":
                    style.top = this.tooltip.clientHeight + "px";
                    style.left = "0";
                    break;
                case "bottom":
                    style.top = (-this.tooltip.clientHeight) + "px";
                    style.left = "0";
                    break;
                case "left":
                    style.top = "0";
                    style.left = this.tooltip.clientWidth + "px";
                    break;
                case "right":
                    style.top = "0";
                    style.left = (-this.tooltip.clientWidth) + "px";
                    break;
            }
        }
    }
    positionTooltip() {
        this.resetTooltipContent();
        if (this.tooltip && this.slottedElement) {
            this.tooltip.style.left = "";
            this.tooltip.style.top = "";
            computeOffsetPosition(this.slottedElement, this.tooltip, this.position)
                .then(({ x, y }) => {
                if (this.tooltip) {
                    this.tooltip.style.left = `${x}px`;
                    this.tooltip.style.top = `${y}px`;
                }
            });
        }
        this.revealDebouncer.run(() => {
            if (this.tooltipContent) {
                this.tooltipContent.style.transition = "top var(--ft-transition-duration, 250ms), " +
                    "left var(--ft-transition-duration, 250ms)";
                this.tooltipContent.style.top = "0";
                this.tooltipContent.style.left = "0";
            }
        }, this.manual ? 0 : this.delay);
    }
    onTouch() {
        if (!this.manual) {
            this.show();
            setTimeout(() => window.addEventListener("touchstart", (e) => {
                if (!e.composedPath().includes(this.container)) {
                    this.onOut();
                }
            }, { once: true }), 100);
        }
    }
    onHover() {
        if (!this.manual) {
            this.show();
        }
    }
    onOut() {
        if (!this.manual) {
            this.revealDebouncer.cancel();
            this.hide();
        }
    }
    onClick() {
        if (!this.manual) {
            this.revealDebouncer.cancel();
            this.hide();
        }
    }
    correctOutOfWindowPixels(leftOrTopOutOfWindowPixels, rightOrBottomOutOfWindowPixels) {
        return Math.max(leftOrTopOutOfWindowPixels, Math.min(0, -rightOrBottomOutOfWindowPixels));
    }
}
FtTooltip.elementDefinitions = {
    "ft-typography": FtTypography,
};
FtTooltip.styles = styles;
__decorate([
    property()
], FtTooltip.prototype, "text", void 0);
__decorate([
    property({ type: Boolean })
], FtTooltip.prototype, "manual", void 0);
__decorate([
    property({ type: Boolean })
], FtTooltip.prototype, "inline", void 0);
__decorate([
    property({ type: Number })
], FtTooltip.prototype, "delay", void 0);
__decorate([
    property()
], FtTooltip.prototype, "position", void 0);
__decorate([
    queryAssignedNodes()
], FtTooltip.prototype, "slotNodes", void 0);
__decorate([
    query(".ft-tooltip--container")
], FtTooltip.prototype, "container", void 0);
__decorate([
    query(".ft-tooltip")
], FtTooltip.prototype, "tooltip", void 0);
__decorate([
    query(".ft-tooltip--content")
], FtTooltip.prototype, "tooltipContent", void 0);
__decorate([
    state()
], FtTooltip.prototype, "visible", void 0);
__decorate([
    eventOptions({ passive: true })
], FtTooltip.prototype, "onTouch", null);
__decorate([
    eventOptions({ passive: true })
], FtTooltip.prototype, "onHover", null);
__decorate([
    eventOptions({ passive: true })
], FtTooltip.prototype, "onOut", null);
__decorate([
    eventOptions({ passive: true })
], FtTooltip.prototype, "onClick", null);
export { FtTooltip };
